@@ -1,5 +1,5 @@
 # server
-from bottle import request, response, FormsDict
+from bottle import request, response, FormsDict, HTTPError
 
 # rest of the project
 from .cleanup import CleanerAgent, CollectorAgent
@@ -46,8 +46,16 @@ import urllib
 dblock = Lock() #global database lock
 dbstatus = {
 	"healthy":False,
-	"rebuildinprogress":False
+	"rebuildinprogress":False,
+	"complete":False
 }
+class DatabaseNotBuilt(HTTPError):
+	def __init__(self):
+		super().__init__(
+			status=503,
+			body="The Maloja Database is still being built. Try again in a few seconds.",
+			headers={"Retry-After":10}
+		)
 
 SCROBBLES = []	# Format: tuple(track_ref,timestamp,saved)
 ARTISTS = []	# Format: artist
@@ -335,33 +343,18 @@ def get_tracks(artist=None):
 
 
 def get_artists():
+	if not dbstatus['healthy']: raise DatabaseNotBuilt()
 	return ARTISTS #well
 
 
 
-
-
-
 def get_charts_artists(**keys):
 	return db_aggregate(by="ARTIST",**{k:keys[k] for k in keys if k in ["since","to","within","timerange"]})
 
 
-
-
-
-
-
 def get_charts_tracks(**keys):
 	return db_aggregate(by="TRACK",**{k:keys[k] for k in keys if k in ["since","to","within","timerange","artist"]})
 
-
-
-
-
-
-
-
-
 def get_pulse(**keys):
 
 	rngs = ranges(**{k:keys[k] for k in keys if k in ["since","to","within","timerange","step","stepn","trail"]})
@@ -373,9 +366,6 @@ def get_pulse(**keys):
 	return results
 
 
-
-
-
 def get_performance(**keys):
 
 	rngs = ranges(**{k:keys[k] for k in keys if k in ["since","to","within","timerange","step","stepn","trail"]})
@@ -401,12 +391,6 @@ def get_performance(**keys):
 	return results
 
 
-
-
-
-
-
-
 def get_top_artists(**keys):
 
 	rngs = ranges(**{k:keys[k] for k in keys if k in ["since","to","within","timerange","step","stepn","trail"]})
@@ -732,6 +716,7 @@ def build_db():
 
 	global dbstatus
 	dbstatus['healthy'] = False
+	dbstatus['complete'] = False
 	dbstatus['rebuildinprogress'] = True
 
 	log("Building database...")
@@ -775,6 +760,8 @@ def build_db():
 			if n % m == 0: log(f"Loaded {n}/{scrobblenum}...")
 
 	if usebar: pbar.done()
+
+
 	log("Database loaded, optimizing...")
 
 	# optimize database
@@ -795,6 +782,9 @@ def build_db():
 	#		ARTISTS.append(artist)
 	# coa.updateIDs(ARTISTS)
 
+	dbstatus['healthy'] = True
+
+
 	#start regular tasks
 	utilities.update_medals()
 	utilities.update_weekly()
@@ -804,7 +794,8 @@ def build_db():
 	global ISSUES
 	ISSUES = check_issues()
 
-	dbstatus['healthy'] = True
+
+	dbstatus['complete'] = True
 	dbstatus['rebuildinprogress'] = False
 
 	log("Database fully built!")
@@ -1027,6 +1018,7 @@ def reduce_caches_if_low_ram():
 # Queries the database
 def db_query_full(artist=None,artists=None,title=None,track=None,since=None,to=None,within=None,timerange=None,associated=False,max_=None):
 
+	if not dbstatus['healthy']: raise DatabaseNotBuilt()
 	(since, to) = time_stamps(since=since,to=to,within=within,range=timerange)
 
 	# this is not meant as a search function. we *can* query the db with a string, but it only works if it matches exactly
@@ -1078,7 +1070,7 @@ def db_query_full(artist=None,artists=None,title=None,track=None,since=None,to=N
 # Queries that... well... aggregate
 def db_aggregate_full(by=None,since=None,to=None,within=None,timerange=None,artist=None):
 
-
+	if not dbstatus['healthy']: raise DatabaseNotBuilt()
 	(since, to) = time_stamps(since=since,to=to,within=within,range=timerange)
 
 	if isinstance(artist, str):
@@ -1194,24 +1186,3 @@ def scrobbles_in_range(start,end,reverse=False):
 			if stamp < start: continue
 			if stamp > end: return
 			yield SCROBBLESDICT[stamp]
-
-
-# for performance testing
-def generateStuff(num=0,pertrack=0,mult=0):
-	import random
-	for _ in range(num):
-		track = random.choice(TRACKS)
-		t = get_track_dict(track)
-		time = random.randint(STAMPS[0],STAMPS[-1])
-		createScrobble(t["artists"],t["title"],time,volatile=True)
-
-	for track in TRACKS:
-		t = get_track_dict(track)
-		for _ in range(pertrack):
-			time = random.randint(STAMPS[0],STAMPS[-1])
-			createScrobble(t["artists"],t["title"],time,volatile=True)
-
-	for scrobble in SCROBBLES:
-		s = get_scrobble_dict(scrobble)
-		for i in range(mult):
-			createScrobble(s["artists"],s["title"],s["time"] - i*500,volatile=True)
